function features = extractFeatures(raw, verbose)
%  Extract the following features:
%
%   - HZCRR : High Zero-Crossing rate Ratio
%   - LSTER : LowSHort Time Energy Ratio
%   - SF : Spectrum Flux
%   - BP : Band Periodicity (4 features)
%   - NFR: Noise Frame Ratio
%
    hanningWindow = hann(200).';

    windows = reshape(raw, 8000, []);

    duration = size(windows, 2);

    lster = zeros(1, duration);
    sf    = zeros(1, duration);
    bp    = zeros(4, duration);
    nfr   = zeros(1, duration); 
    hzcrr = zeros(1, duration);


    for v = 1:duration
        if (verbose)
            fprintf('%d / %d = %.2f %% \n', v, duration, v / duration * 100);
        end
        window = windows(:, v);
        frames = reshape(window, 200, 40);

        ZCRs = zeros(1, 40);
        STEs = zeros(1, 40);
        SFs  = zeros(1, 39);
        NFRs = zeros(1, 40);

        for f = 1:size(frames, 2)
            frame = frames(:, f).';

            ZCRs(f)  = ZCR(frame); 
            % STE
            STEs(f) = STE(frame);
            
            NFRs(f) = NFR(frame);
            
            % SF
            wav_window = hanningWindow .* frame;
            window_length = 200;
            FFT = abs(fft(wav_window, 2*window_length));
            FFT = FFT(1 : window_length);
            if (max(FFT) ~= 0)
                FFT = FFT/max(FFT);            
            end
            %spec = fft(frame .* hanningWindow);
            if f > 1
                SFs(f-1) = SF(FFT', prevFFT');
            end
            prevFFT = FFT;

        end
        % subband energies (BP1 BP2 BP3 BP4)
        for i = 1:4
            s = subband(window, i);
            sframes = reshape(s, 200, 40);
            fbps = zeros(39,1);
            for f = 1:39
                subbandFrame = [sframes(:, f); sframes(:, f+1)];
                fbps(f) = BP(subbandFrame);
            end
            bp(i, v) = mean(fbps);
        end

        hzcrr(v) = mean(sign(ZCRs - ones(1, 40) * 1.5 * mean(ZCRs))+1) / 2;
        lster(v) = sum(sign(ones(1,40) * 0.5 * mean(STEs) - STEs)+1) / 2 / 40;
        sf(v)    = sum(SFs) / length(SFs);
        nfr(v)   = mean(NFRs < 0.32);
    end

    features = [hzcrr' lster' sf' bp' nfr'];
    
    features(isnan(features) | isinf(features)) = 0;
end

function y = ZCR(x)
    % Zero-Crossing rate con threshold
    epsilon = 0.1;
    % Gaston version
    %y = (sum(abs(diff(x > epsilon))) + sum(abs(diff(x < -epsilon)))) / 2;
    % New version
    y = sum(abs(diff((x > epsilon) - (x < -epsilon))))/2 / length(x);
    % Original version
    %y = sum(abs(diff(x>0)))/length(x);
end

function ste = STE(audio)
% Short Time Energy
    ste = mean(audio.^2);
end

function bp = BP(s)
% Band Periodicity
% s = subband
    if size(s, 2) > 1
        s = s.';
    end
    m = length(s) / 2;
    v2 = s(m+1:end);
    normv2 = norm(v2);
    
    crossCorrelation = xcorr(v2,s);
    crossCorrelation = crossCorrelation(m+1:2*m);
    denom = conv(s.^2, ones(1,m));
    denom = denom(m:2*m-1);
    denom = flipud(sqrt(denom));
    r = crossCorrelation ./ denom / normv2;
    bp = max(r);

end

function s = subband(audio, i)
% ith subband
    % create the filterbank
    limits = [500 1000; 1000 2000; 2000 3000; 3000 3999];
    
    f = fir1(30,  limits(i,:) ./ 4000, 'bandpass'); 

    s = filter(f, 1, audio);
end

function sf = SF(frameSpectrum, prevFrameSpectrum)
% Spectrum Flux
    %sf = norm(abs(frameSpectrum)-abs(prevFrameSpectrum));
% difference spectrum (set first diff to zero)
    %sf = norm(abs(frameSpectrum-prevFrameSpectrum));
    sf = sqrt(sum((frameSpectrum - prevFrameSpectrum) .^ 2));
end

function nfr = NFR(frame)
% Noise Frame Ratio
   autocorr = xcorr(frame, 'coeff');
   autocorr(length(frame)+(-5:5)) = 0; % eliminate the central peak
   nfr = max(abs(autocorr));
end


